-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Script:  HeliGyro
-- Version: 1.0  RC1
-- Build:   2015-01-20
-- By:      oe3gsu@x-plane.at
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Description:
-- Simulates a Gyro-controll of the yaw of your helo
-- Usable in all helos, not working if your aircraft is not in the "Aircraft/Helicopters" - folder!
-- Display if HUD - addition only if view is "forward with HUD"
-- Stabilization parameters can be set with configuration file:
--   store a textfile with the same name as the .acf - file with extension .hgc in the aircraft folder
--   an example can be found at the "Scripts"-folder
--   edit only the numbers in the same format found in the example, leave comments unchanged!
--
-- INSTALLATION: 
--     copy the file "HeliGyro.lua" and "HeliGyro_std.hgc" into your "Resources/plugins/FlyWithLua/Scripts"-Folder
--     copy an new created .hgc-file to the aircraft-folder of your helo
--     load your helo and have fun
--
-- INSTRUCTIONS:
--    At startup the gyro-system is off! 
--    You can see a red dot and the loaded config-values (for 30 seconds)
--    If there is no config file in the aircraft-folder, defaults from "Scripts"-folder will be used
--    First assign a key or button to "FlyWithLua/HeliGyro/switch_Mode"
--    Press this key or button and the dot will get green: Gyro System is ON
--    Fly and have fun!
-- 
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

-- USER editable values -------------------------------------------------------------------------------
--                                                                                                   --
--                 there are no, i repeat: NO user-editable values in this script !!!                --
--                   all changeable values you can modify in the configuration file                  --
--                                 PLEASE READ THE MANUAL                                            --
--                                                                                                   --
-- END     DONT edit anything below this line !!! -----------------------------------------------------

-- STARTUP --------------------------------------------------------------------------

require("graphics")

DataRef("HG_xp_rud_trim", "sim/flightmodel/controls/rud_trim", "writable")
DataRef("HG_xp_joy_yaw", "sim/joystick/yoke_heading_ratio", "readonly")
DataRef("HG_xp_speed", "sim/flightmodel/position/indicated_airspeed2", "readonly")
DataRef("HG_xp_true_psi", "sim/flightmodel/position/true_psi", "readonly")
DataRef("HG_xp_yaw_force", "sim/flightmodel/forces/N_total", "writable")
DataRef("HG_xp_yaw_beta", "sim/flightmodel/position/beta", "readonly")
DataRef("HG_xp_elev_agl", "sim/flightmodel/position/y_agl", "readonly")
DataRef("HG_xp_vert_spd", "sim/flightmodel/position/vh_ind", "readonly")
DataRef("HG_xp_hud_brht", "sim/cockpit/electrical/HUD_brightness", "readonly")
DataRef("HG_xp_view_type", "sim/graphics/view/view_type", "readonly")

local HG_debug = false
HG_SW_MODE = false
local HG_MODE = 0
local HG_head_diff = 0
local HG_head_soll = HG_xp_true_psi
local HG_trim_factor = 1.0
local HH_corr = 0.0
local HG_speed_corr = 0.0
local HG_start_time = os.clock()

local HG_sensitivity  = 1.0
local HG_min_speed  = 10.0
local HG_max_speed  = 40.0
local HG_joy_input = 1.0
local HG_joy_deadz = 0.05


-- local 
local HG_parm = {HG_sensitivity, HG_min_speed, HG_max_speed, HG_joy_input, HG_joy_deadz}



function HG_load_settings() -------------------------------------------------------- LOAD SETTINGS
  
  if string.match(AIRCRAFT_PATH, "Heli") then 
    
    file = io.open(AIRCRAFT_PATH .. string.sub(AIRCRAFT_FILENAME, 1, string.len(AIRCRAFT_FILENAME)- 4) .. ".hgc", "r")
    if file == nil then file = io.open(SCRIPT_DIRECTORY .. "HeliGyro_std.hgc", "r") end
   
    for i=1, table.getn(HG_parm) do
      rd = file:read("*line")
      -- HG_parm[i] = rd
      HG_parm[i] = string.sub(rd, 1, string.find(rd, " "))
      i = i+1
      if rd == nil then 
        file:close()
        break 
      end
    end
    
    HG_sensitivity  = tonumber(HG_parm[1])
    HG_min_speed  = tonumber(HG_parm[2])
    HG_max_speed  = tonumber(HG_parm[3])
    HG_joy_input = tonumber(HG_parm[4])
    HG_joy_deadz = tonumber(HG_parm[5])
  
  end
end -----------------------------------------------------------

HG_load_settings()

function HG_show_display() --------------------------------------------------------SHOW Display

  if string.match(AIRCRAFT_PATH, "Heli") then 
    
    if HG_xp_view_type ~= 1023 then return end ------------------ Show only if view is forward with HUD
      
      XPLMSetGraphicsState(0,0,0,1,1,0,0)
      local window_x = SCREEN_WIDTH / 2
      local window_y = SCREEN_HIGHT - 20
      
      
      graphics.set_color(0.3, 1.0, 0.0, HG_xp_hud_brht)
      graphics.draw_line(window_x-250, window_y+10, window_x+250, window_y+10)
      graphics.draw_line(window_x+250, window_y+10, window_x+250, window_y-10)
      graphics.draw_line(window_x+250, window_y-10, window_x-250, window_y-10)
      graphics.draw_line(window_x-250, window_y+10, window_x-250, window_y-10)
      
      graphics.draw_line(window_x+250, window_y-10,  window_x+250, window_y-310)
      graphics.draw_line(window_x+250, window_y-310, window_x+270, window_y-310)
      graphics.draw_line(window_x+270, window_y-310, window_x+270, window_y-10)
      graphics.draw_line(window_x+270, window_y-10,  window_x+250, window_y-10)
      graphics.draw_line(window_x+250, window_y-160, window_x+270, window_y-160)
      
      local HG_vert_spd = HG_xp_vert_spd * 15
      if HG_vert_spd >  150 then HG_vert_spd =  150 end
      if HG_vert_spd < -150 then HG_vert_spd = -150 end
      graphics.draw_filled_circle(window_x+260, window_y-160+HG_vert_spd, 7)
      draw_string_Helvetica_18(window_x-350, window_y-7, "AGL")
      draw_string_Helvetica_18(window_x+300, window_y-7, "V.Speed")
      draw_string_Helvetica_18(window_x-290, window_y-7, string.format("%5.1f", HG_xp_elev_agl))
      if HG_xp_vert_spd > 0 then graphics.set_color(0.3, 1.0, 0.0, HG_xp_hud_brht) else graphics.set_color(1.0, 0.3, 0.0, HG_xp_hud_brht) end
      draw_string_Helvetica_18(window_x+270, window_y-7, string.format("%3.1f", HG_xp_vert_spd))
      
      if HH_corr < 0 then
        graphics.set_color(1.0, 0.0, 0.0, HG_xp_hud_brht)    
        graphics.draw_rectangle(window_x+(HH_corr), window_y+5, window_x, window_y-5)
      else
        graphics.set_color(0.0, 1.0, 0.0, HG_xp_hud_brht)    
        graphics.draw_rectangle(window_x, window_y+5, window_x+(HH_corr), window_y-5)
      end
      
      if HG_MODE == 0 then
        graphics.set_color(1.0, 0.0, 0.0, HG_xp_hud_brht)
      end
      if HG_MODE == 1 then
        graphics.set_color(0.0, 1.0, 0.0, HG_xp_hud_brht)
      end
      local HG_yaw_beta = HG_xp_yaw_beta
      if HG_yaw_beta < -250 then HG_yaw_beta = -250 end
      if HG_yaw_beta >  250 then HG_yaw_beta =  250 end
      graphics.draw_filled_circle(window_x-HG_yaw_beta, window_y, 9)
      
      if os.clock() < HG_start_time + 30 then
        graphics.set_color(1.0, 1.0, 1.0, 1.0)    
        draw_string_Helvetica_12(window_x-5, window_y-25, HG_sensitivity)
        draw_string_Helvetica_12(window_x-5, window_y-45, HG_min_speed)
        draw_string_Helvetica_12(window_x-5, window_y-65, HG_max_speed)
        draw_string_Helvetica_12(window_x-5, window_y-85, HG_joy_input)
        draw_string_Helvetica_12(window_x-5, window_y-105, HG_joy_deadz)
      end
      
      if HG_debug then
      graphics.set_color(0.5, 0.5, 0.5, 0.5)    
      graphics.draw_rectangle(window_x+210, window_y+18, window_x+550, window_y-150)
      graphics.set_color(1.0, 1.0, 1.0, 1.0)    
      draw_string_Helvetica_12(window_x+250, window_y+5, "YAW-Trim")
      draw_string_Helvetica_12(window_x+350, window_y+5, string.format("%3.3f", HG_xp_rud_trim))
      draw_string_Helvetica_12(window_x+250, window_y-20, "YAW-Joy")
      draw_string_Helvetica_12(window_x+350, window_y-20, string.format("%3.3f",  HG_xp_joy_yaw))
      draw_string_Helvetica_12(window_x+250, window_y-40, "HG_speed_corr")
      draw_string_Helvetica_12(window_x+350, window_y-40, string.format("%3.3f",  HG_speed_corr))
      draw_string_Helvetica_12(window_x+250, window_y-60, "HeadingSoll")
      draw_string_Helvetica_12(window_x+350, window_y-60, string.format("%3.3f",  HG_head_soll))
      draw_string_Helvetica_12(window_x+250, window_y-80, "Airspeed")
      draw_string_Helvetica_12(window_x+350, window_y-80, string.format("%3.3f",  HG_xp_speed))
      draw_string_Helvetica_12(window_x+250, window_y-100, "HH_corr")
      draw_string_Helvetica_12(window_x+350, window_y-100, string.format("%3.3f",  HH_corr))
      
    end
    
  end

end -- function --------------------------------------------------------------------------

do_every_draw("HG_show_display()")

function HG_switch_mode() --------------------------------------------------------------SWITCH MODE
  if string.match(AIRCRAFT_PATH, "Heli") then 
  
    if HG_SW_MODE == true then
      HG_MODE = HG_MODE + 1
      if HG_MODE > 1 then HG_MODE = 0 end
      if HG_MODE == 1 and HG_SW_MODE == true then  HG_head_soll = HG_xp_true_psi end   -- store act. heading in HG_head_soll
      HG_SW_MODE = false
    end
  end
end

do_every_draw("HG_switch_mode()")

function HG_main() ----------------------------------------------------------------- MAIN
  if string.match(AIRCRAFT_PATH, "Heli") then 
  
    if HG_MODE == 0 then 
      HG_xp_rud_trim = 0 
     HG_head_soll = HG_xp_true_psi
    end
    
    if HG_MODE == 1 then
      
      -- HG_head_soll = HG_head_soll - (HG_head_diff * (1 - HG_speed_corr))            --Heading Corr. when speed > 10 
      -- if HG_xp_speed > 40 then HG_head_soll = HG_head_soll end -- HG_head_diff end
      
      HG_head_diff = (HG_head_soll - HG_xp_true_psi)                                   -- calculate heading difference
      if HG_head_diff > 180 then HG_head_diff = HG_head_diff -360 end
      if HG_head_diff < -180 then HG_head_diff = HG_head_diff +360 end
      
      -- HG_speed_corr = ((30 - HG_xp_speed) / 30) + 0.33                              -- calculate speed correction
      HG_speed_corr = (HG_max_speed - HG_min_speed - HG_xp_speed) / (HG_max_speed - HG_min_speed) + 1/((HG_max_speed - HG_min_speed)/HG_min_speed)
      if HG_speed_corr > 1 then HG_speed_corr = 1 end
      if HG_speed_corr < 0 then HG_speed_corr = 0 end
      
      HH_corr = HG_head_diff * (HG_speed_corr * HG_sensitivity)                        -- calculate correcture 
      
      HG_xp_rud_trim = (HH_corr/10)                                                    -- apply correcture
      
      if HG_xp_rud_trim > 1.0 then HG_xp_rud_trim = 1.0 end                            -- trim the yaw-trim values
      if HG_xp_rud_trim < -1.0 then HG_xp_rud_trim = -1.0 end 
      
      if HG_speed_corr == 0 then                                                       -- correct HG_head_soll while cruising
        HG_head_soll = HG_head_soll - (HG_head_diff / 100)                             -- do it slow :-)
      end
      
      if math.abs(HG_xp_joy_yaw) > HG_joy_deadz then HG_head_soll = HG_head_soll + (HG_xp_joy_yaw * HG_joy_input) end   -- Joystick YAW - Input
      if HG_head_soll > 360 then HG_head_soll = HG_head_soll -360 end
      if HG_head_soll < 0 then HG_head_soll = HG_head_soll +360 end
      
    end
  end

end

do_every_frame("HG_main()")

create_command("FlyWithLua/HeliGyro/switch_Mode", "Switch Gyro-Mode", "HG_SW_MODE = true", "", "")
